#!/usr/bin/python

import ctypes
import os
import threading


BOOLEAN = ctypes.c_ubyte
FALSE = 0
TRUE = 1
# single chan regular token
CMND_SINGLE_CHAN_SCAN = 0x89 
# single chan with digin token
CMND_SINGLE_CHAN_DIGIN_SCAN = 0x8D  
# multi chan regular token
CMND_MULTI_CHAN_SCAN = 0x8B 
# multi chan with cal token
CMND_MULTI_CHAN_CAL_SCAN = 0x8C 
# multi chan with digin token	
CMND_MULTI_CHAN_DIGIN_SCAN = 0x8E 
# multi chan with cal and digin token
CMND_MULTI_CHAN_CAL_DIGIN_SCAN = 0x8F 


def funcDoScan():
  global LL_USB_Lib

  class scanStruct(ctypes.Structure):
     _fields_ = [('fGotBuffWrap', ctypes.c_ubyte), ('inThread', ctypes.c_int),
       ('isScanning', ctypes.c_int), ('fScanThreadLoopDone', ctypes.c_ubyte),
       ('scanBuff', ctypes.POINTER(ctypes.c_int)),
       ('diginBuff', ctypes.POINTER(ctypes.c_ubyte)),
       ('uiStackedScans', ctypes.c_uint), ('uiHeadPntr', ctypes.c_uint),
       ('uiTailPntr', ctypes.c_uint), ('uiBufferSizeMin', ctypes.c_uint),
       ('uiBufferSizeMax', ctypes.c_uint), ('uiScanToken', ctypes.c_uint),
       ('uiScanChans', ctypes.c_uint), ('uiPointsInBuff', ctypes.c_uint)]


  ## scan struct variable init start  ===
  testByte = ctypes.c_ubyte(6)
  LL_USB_Lib.testTempStruct.argtypes = [ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.testTempStruct.restypes = ctypes.c_ubyte

  scanStruct = scanStruct.in_dll(LL_USB_Lib, 'scanStruct')

  scanStruct.inThread = 1
  scanStruct.uiBufferSizeMin = 500; scanStruct.uiBufferSizeMax = 500
  scanStruct.uiHeadPntr = 0; scanStruct.uiTailPntr = 0
  scanStruct.isScanning = FALSE
  scanStruct.fGotBuffWrap = FALSE 
  scanStruct.fScanThreadLoopDone = FALSE  
  scanStruct.uiScanToken = CMND_SINGLE_CHAN_SCAN
  scanStruct.uiScanChans = 1
  scanStruct.uiStackedScans = 1 #MUST ALWAYS BE AT LEAST 1
  ## scan struct variable init end  ===



  #following just defaults, will be asked for both when the scan is started
  #search "doSetNumberOfScans" for explanation using each of the following
  fContinuousScan = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fContinuousScan')
  fContinuousScan.value = TRUE	#this trumps the others. If true, uiScansToDo is ignored	
  fTimedScan = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fTimedScan')
  fTimedScan.value = FALSE
  uiScansToDo = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScansToDo')
  uiScansToDo.value = 100		#"scans" not data points (default to 100)
  fDoDispData = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fDoDispData')
  fDoDispData.value = TRUE
  fContScanDone = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fContScanDone')
  fContScanDone.value = FALSE 
  fMainScanLoopBusy = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fMainScanLoopBusy')
  fMainScanLoopBusy.value = TRUE 
  fEndScanTokenSent = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fEndScanTokenSent')
  fEndScanTokenSent.value = FALSE 
  fScanThreadActive = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanThreadActive')
  fScanThreadActive.value = FALSE 

  uiScanHours = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanHours')
  uiScanHours.value = 1		#just any default
  fScanHours = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanHours')
  fScanHours.value = FALSE		#just any default 
  uiScanMinutes = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanMinutes')
  uiScanMinutes.value = 1		#just any default 
  fScanMinutes = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanMinutes')
  fScanMinutes.value = FALSE	#just any default 
  uiScanSeconds = ctypes.c_uint.in_dll(LL_USB_Lib, 'uiScanSeconds')
  uiScanSeconds.value = 60		#just any default 
  fScanSeconds = ctypes.c_ubyte.in_dll(LL_USB_Lib, 'fScanSeconds')
  fScanSeconds.value = FALSE	#just any default 


  fIsDigScan = ctypes.c_ubyte(0); fIsDigScan.value = FALSE
  if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN) 
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_DIGIN_SCAN)
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN)):
      fIsDigScan.value = TRUE

  bAddForOffsetChan = ctypes.c_ubyte(0); bAddForOffsetChan.value = 0
  if((scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_SCAN)
    or (scanStruct.uiScanToken == CMND_MULTI_CHAN_CAL_DIGIN_SCAN)):
    bAddForOffsetChan = 1;
  bIsSingleChan = ctypes.c_ubyte(0); bIsSingleChan = 0
  if((scanStruct.uiScanToken == CMND_SINGLE_CHAN_SCAN) 
      or (scanStruct.uiScanToken == CMND_SINGLE_CHAN_DIGIN_SCAN)):
    bIsSingleChan = 1

  scanStruct.uiBufferSizeMax = (scanStruct.uiBufferSizeMin * scanStruct.uiStackedScans * 4)
  print ("scanStruct.uiBufferSizeMax: %d\n" % scanStruct.uiBufferSizeMax)
  scanStruct.scanBuff = (ctypes.c_int * (scanStruct.uiBufferSizeMax))() #allocate buffer
  for x in range(0, (scanStruct.uiBufferSizeMax)):
    scanStruct.scanBuff[x] = 2



  ##################################################
  ## start of thread ###############################
  LL_USB_Lib.doScan.argtypes = [ctypes.c_void_p]
  LL_USB_Lib.doScan.restypes = ctypes.c_void_p
  testVal = ctypes.c_void_p(6) #made 6 but can be any val since it's not used

  scanThread = threading.Thread(target=LL_USB_Lib.doScan, args=(testVal,))
  scanThread.start()
  print ("here comes the data\n")

  uiScansToGet = 0

  while(1==1):
    if((scanStruct.uiPointsInBuff) > 1):
      if uiScansToGet < 10:
        print ("scan: {:d}  volts: {:f}".format(uiScansToGet, float((scanStruct.scanBuff[scanStruct.uiTailPntr] - 8388608)) * 0.00000059604645))
      else:
        print ("scan: {:d} volts: {:f}".format(uiScansToGet, float((scanStruct.scanBuff[scanStruct.uiTailPntr] - 8388608)) * 0.00000059604645))
        
      scanStruct.uiTailPntr+=1
      if scanStruct.uiTailPntr == scanStruct.uiBufferSizeMax:
        print ("zeroing tail pointer\n")
        scanStruct.uiTailPntr = 0

      scanStruct.uiPointsInBuff -= 1
      uiScansToGet += 1
      if uiScansToGet == 25:
        fContScanDone.value = TRUE 
        fMainScanLoopBusy.value = FALSE
        while(fEndScanTokenSent.value == FALSE):
          pass

        return			 



def main():
  global LL_USB_Lib

  #initialize library
  LL_USB_Lib = ctypes.cdll.LoadLibrary('/usr/lib/libLL_USB30x.so')
  usDevID = ctypes.c_ushort(5206)
  os.system("clear")

  rate = float(100.0)
  LL_USB_Lib.EX_ConnectOneDevice.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]

  LL_USB_Lib.EX_ConnectOneDevice.restypes = ctypes.c_byte
  print "\nsigning on (may take a while) - WAIT . . ."
  if LL_USB_Lib.EX_ConnectOneDevice(usDevID, rate, 3):
    print "  signed on\n"
  else:
    print "not signed on\n"
    return

  dblMilliVolts = ctypes.c_double(100)
  print "getting a conversion - WAIT . . ."
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan0: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return

  bChan = ctypes.c_ubyte(6)
  bLastDigin = ctypes.c_ubyte(0)
  print "changing to  full-scale channel 6 - WAIT . . ."
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(usDevID, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    return

  print "getting a conversion - WAIT . . ."
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts)	
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan6: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return


  bDAC = ctypes.c_ubyte(0)
  dblDACVolts = ctypes.c_double(3.5)
  print "setting DAC0 to 3.5 volts - WAIT . . ."
  LL_USB_Lib.EX_SendDAC.argtypes = [ctypes.c_ushort, ctypes.c_double, ctypes.c_ubyte]
  LL_USB_Lib.EX_SendDAC.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendDAC(usDevID, dblDACVolts, bDAC):	
    print ("  sendDAC success")
  else:
    print ("  failed setting DAC\n")
    return  

  bChan = 3
  print "changing to channel 3 (should be DAC0 volts)"
  print "   see pythonSampleAppMini.pdf for how to connect - WAIT . . ."
  LL_USB_Lib.EX_SendChan.argtypes = [ctypes.c_ushort, ctypes.c_ubyte, ctypes.POINTER(ctypes.c_ubyte)]
  LL_USB_Lib.EX_SendChan.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_SendChan(usDevID, bChan, bLastDigin):	
    print ("  sendChan success, lastDigin: %d\n" % bLastDigin.value)
  else:
    print ("  failed sending channel\n")
    return

  print "getting a conversion - WAIT . . ."
  LL_USB_Lib.EX_GetOneConversion.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetOneConversion.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetOneConversion(usDevID, dblMilliVolts):	
    print ("  got volts chan3: %f\n" % dblMilliVolts.value)
  else:
    print ("  failed reading conversion\n")
    return

  dblActualRate = ctypes.c_double(1000)
  print "checking 'actual rate' when 1000Hz is requested - WAIT . . ."
  LL_USB_Lib.EX_GetCalculatedRate.argtypes = [ctypes.c_ushort, ctypes.POINTER(ctypes.c_double)]
  LL_USB_Lib.EX_GetCalculatedRate.restypes = ctypes.c_ubyte
  if LL_USB_Lib.EX_GetCalculatedRate(usDevID, dblActualRate):	
    print ("  1000Hz would be: %f\n" % dblActualRate.value)
  else:
    print ("  failed getting actual rate for 1000Hz\n")
    return
  LL_USB_Lib.AllowPrintf(1) #allows printf of some notifications from within library

  print ("starting scan")
  funcDoScan()
  print ("done scan")

  return


if __name__ == '__main__':
  main()
